import 'dart:async';
import 'dart:developer';
import 'dart:io';
import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter_image_compress/flutter_image_compress.dart';
import 'package:flutter_video_compress/flutter_video_compress.dart';
import 'package:image_picker/image_picker.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_media/model/nui_media_file.dart';
import 'package:nui_media/ui/dialog/nui_select_media_dialog.dart';
import 'package:photo_manager/photo_manager.dart';
import 'package:path_provider/path_provider.dart' as path_provider;
import 'package:path/path.dart' as path;
import 'package:flutter/material.dart';
import 'package:better_player/better_player.dart';
import 'package:video_player/video_player.dart';
import 'package:video_thumbnail/video_thumbnail.dart';

enum MediaType{
  PNG,
  JPG,
  MP4,
  MP4_BIT
}

enum MediaSelectionOption{
  GALLERY,
  CAMERA,
  BOTH
}

typedef MediaSelectionListener = void Function(List<NUIMediaFile> medias);

class NUIVideoInfo{
  NUIVideoInfo({this.size, this.width, this.height, this.duration});
  int size;
  double width;
  double height;
  Duration duration;
}

class NUIImageInfo{
  NUIImageInfo({this.size, this.width, this.height});
  int size;
  double width;
  double height;
}

List<NUIMediaFile> showMediaSelection(
    BuildContext context, {
    String title,
    TextStyle titleStyle,
    TextStyle selectionStyle,
    Color iconColor,
    Widget galleryIcon,
    Widget cameraIcon,
    Widget recordIcon,
    bool enableGallery: true,
    bool enableCamera: true,
    bool enableRecord: true}){
  NUIMediaUtil.showMediaSelection(context, title: title, titleStyle: titleStyle, selectionStyle: selectionStyle, iconColor: iconColor, galleryIcon: galleryIcon, cameraIcon: cameraIcon, recordIcon: recordIcon, enableGallery: enableGallery, enableCamera: enableCamera, enableRecord: enableRecord);
}

class NUIMediaUtil{
  NUIMediaUtil._builder();

  static Future<List<AssetEntity>> getAllFromGallery({RequestType type: RequestType.all, int limit:100}) async {
    var result = await PhotoManager.requestPermission();
    if(result){
      List<AssetEntity> finalImageList = List();
      List<AssetPathEntity> list = await PhotoManager.getAssetPathList(type: type);
      int galleryCount = 0;
      for(AssetPathEntity ape in list){
        final intRemainingSlot = limit - galleryCount;
        if(intRemainingSlot > 0) {
          final imageList = await ape.getAssetListRange(start: galleryCount, end: galleryCount + intRemainingSlot);
          finalImageList.addAll(imageList);
          galleryCount = finalImageList.length;
        }
      }
      return finalImageList;
    }
    else{
      return null;
    }
  }

  static Future<List<NUIMediaFile>> getAllFromGalleryAsNUIMedias({RequestType type: RequestType.all, int limit:100}) async {
    var result = await PhotoManager.requestPermission();
    if(result){
      List<NUIMediaFile> finalImageList = List();
      List<AssetPathEntity> list = await PhotoManager.getAssetPathList(type: type);
      int galleryCount = 0;
      for(AssetPathEntity ape in list){
        final intRemainingSlot = limit - galleryCount;
        if(intRemainingSlot > 0) {
          final imageList = await ape.getAssetListRange(start: galleryCount, end: galleryCount + intRemainingSlot);
          imageList.forEach((element) async{
            final path = (await element.file).path;
            finalImageList.add(NUIMediaFile(path: path, isLocal: true, width: element.width, height: element.height, thumbData: element.type == AssetType.video ? element.thumbData : null, videoPath: element.type == AssetType.video ? path : null));
          });
          galleryCount = finalImageList.length;
        }
      }
      return finalImageList;
    }
    else{
      return null;
    }
  }

  static Future<AssetEntity> getMediaById(String id) async{
    return await AssetEntity.fromId(id);
  }

  static Future<File> compressImage(File file, {String targetPath, int minWidth:1080, int minHeight: 1080, int quality: 90}) async {
    var result = await FlutterImageCompress.compressAndGetFile(
      file.absolute.path, targetPath ?? (await copyTempFile(file)).path,
      quality: quality ?? 90,
      autoCorrectionAngle: true,
      minHeight: minHeight,
      minWidth: minWidth
    );

    log("NUMedialUtil -> Compress image source : ${file.lengthSync()}");
    log("NUIMediaUtil -> Compress image result : ${result.lengthSync()}");
    return result;
  }

  static Future<NUIMediaFile> compressVideoWithBitmap(File file, {String targetPath, int quality, bool deleteOrigin: false, bool includeAudio: true}) async{
    final mediaFile = NUIMediaFile(path: null, width: null, height: null);
    final bitmap = bitmapFromVideo(file.path, ImageFormat.PNG);
    final video = compressVideo(file, targetPath: targetPath, quality: quality, deleteOrigin: deleteOrigin, includeAudio: includeAudio);
    final bitmapResult = await bitmap;
    final videoResult = await video;
    mediaFile.path = bitmapResult.path;
    mediaFile.videoPath = videoResult.path;
    mediaFile.isLocal = true;
    return mediaFile;
  }

  static Future<MediaInfo> compressVideo(File file, {String targetPath, int quality, bool deleteOrigin: false, bool includeAudio: true}) async{
    final vdCompress = FlutterVideoCompress();
    final media = await vdCompress.compressVideo(file.path, deleteOrigin: deleteOrigin, includeAudio: includeAudio);
    return media;
  }
  
  static Future<File> copyTempFile(File file) async{
    return getFile(nameFromPath(file.path));
  }

  static String nameFromPath(String filePath, {bool withExtension: true}){
    if(withExtension == true) {
      return path.basename(filePath);
    }
    return path.basenameWithoutExtension(filePath);
  }

  static Future<File> bitmapFromVideo(String path, ImageFormat format, {int maxWidth, int maxHeight, quality: 90}) async{
    final targetPath = await getFilePath(NUIDTUtil.dateToString(DateTime.now(), format: NUIDTUtil.FORMAT_dd_MM_yyyy_dash) + ".png");
    await VideoThumbnail.thumbnailFile(video: path, thumbnailPath: targetPath, imageFormat: format, maxHeight: maxHeight, maxWidth: maxWidth, quality: quality);
    return File(targetPath);
  }
  
  static Future<File> getOutputMediaFile(MediaType type, {bool createFirst: true}) async{
    // Create activity media file name
    final uuid = randomUUID().replaceAll("-","").substring(0, 5);
    final timeStamp = NUIDTUtil.dateToString(DateTime.now(), format: NUIDTUtil.FORMAT_yyyyMMdd_HHmmss) + "_" + uuid;

    String filename;
    if (type == MediaType.JPG) {
      filename = ("IMG_" + timeStamp + ".jpg");
    }
    else if (type == MediaType.MP4) {
      filename = ("VID_" + timeStamp + ".mp4");
    }
    else if (type == MediaType.MP4_BIT){
      filename = ("VID_" + timeStamp + "_video.jpg");
    }
    else if(type == MediaType.PNG){
      filename = ("IMG_" + timeStamp + ".png");
    }
    else{
      filename = ("IMG_" + timeStamp + ".jpg");
    }

    if(createFirst){
      return getFile(filename);
    }
    else{
      return File(await getFilePath(filename));
    }
    //return mediaFile;
  }

  static Future<Directory> getTemporaryDirectory() async {
    return Directory.systemTemp;
  }

  static Future<String> getFilePath(String filename) async {
    final dir = await path_provider.getTemporaryDirectory();
    return "${dir.absolute.path}/$filename";
  }

  static Future<File> getFile(String filename) async {
    final dir = await path_provider.getTemporaryDirectory();
    File file = createFile("${dir.absolute.path}/$filename");
    return file;
  }

  static File createFile(String path) {
    final file = File(path);
    if (!file.existsSync()) {
      file.createSync(recursive: true);
    }
    return file;
  }

  static Future<NUIImageInfo> getImageInfo(String imagePath, {bool local: true}) async {
    Completer<NUIImageInfo> completer = Completer();
    Image image = local ? Image.file(File(imagePath)) : Image.network(imagePath);
    image.image.resolve(ImageConfiguration()).addListener(
      ImageStreamListener((ImageInfo image, bool synchronousCall) async{
          var myImage = image.image;
          final fileSize = local ? await File(imagePath).length() : null;
          Size size = Size(myImage.width.toDouble(), myImage.height.toDouble());
          completer.complete(NUIImageInfo(size: fileSize, width: size.width, height: size.height));
        },
      ),
    );
    return completer.future;
  }

  static Future<NUIVideoInfo> getVideoInfo(String videoPath) async{
    final file = File(videoPath);
    final controller = VideoPlayerController.file(file);
    final duration = controller.value.duration;
    final width = controller.value.size.width;
    final height = controller.value.size.height;
    final size = await file.length();
    return NUIVideoInfo(size: size, duration: duration, width: width, height: height);
  }

  static Future<NUIMediaFile> fileToMediaFile(File file, {bool local: true, MediaType type}) async{
    switch(type){
      case MediaType.JPG:
      case MediaType.MP4_BIT:
      case MediaType.PNG: {
        final imageInfo = await getImageInfo(file.path, local: local);
        return NUIMediaFile(path: file.path, width: imageInfo.width.toInt(), height: imageInfo.height.toInt());
      }
      case MediaType.MP4: {
        final videoInfo = await getVideoInfo(file.path);
        final mediaFile = NUIMediaFile(path: file.path, width: videoInfo.width.toInt(), height: videoInfo.height.toInt());
        mediaFile.videoDuration = videoInfo.duration.inMilliseconds;
        return mediaFile;
      }
      default: {
        final imageInfo = await getImageInfo(file.path, local: local);
        return NUIMediaFile(path: file.path, width: imageInfo.width.toInt(), height: imageInfo.height.toInt());
      }
    }
  }

  static Future<NUIMediaFile> chooseImageFromGallery({ImageSource source: ImageSource.gallery, int imageQuality : 100, double maxWidth, double maxHeight}) async{
    final ImagePicker imagePicker = ImagePicker();
    PickedFile image = await  imagePicker.getImage(
        source: source ?? ImageSource.gallery,
        imageQuality: imageQuality ?? 100,
        maxWidth: maxWidth,
        maxHeight: maxHeight
    );
    if(image != null) {
      return fileToMediaFile(File(image.path), local: true, type: MediaType.PNG);
    }
    return null;
  }

  static Future<NUIMediaFile> chooseVideoFromGallery({ImageSource source: ImageSource.gallery, Duration maxDuration}) async{
    final ImagePicker imagePicker = ImagePicker();
    PickedFile image = await  imagePicker.getVideo(
        source: source ?? ImageSource.gallery,
        maxDuration: maxDuration
    );
    return fileToMediaFile(File(image.path), local: true, type: MediaType.MP4);
  }

  static NUIBottomSheet showMediaSelection(
      BuildContext context, {
        String title,
        TextStyle titleStyle,
        TextStyle selectionStyle,
        Color iconColor,
        Widget galleryIcon,
        Widget cameraIcon,
        Widget recordIcon,
        bool enableGallery: true,
        bool enableCamera: true,
        bool enableRecord: true,
        MediaSelectionListener listener,
      }){
    final dialog = showNUIBottomSheet(context: context, dismissible: true, content: (context) => NUISelectMediaDialogUI(title: title, titleStyle: titleStyle, selectionStyle: selectionStyle, iconColor: iconColor, galleryIcon: galleryIcon, cameraIcon: cameraIcon, recordIcon: recordIcon, enableCamera: enableCamera, enableRecord: enableRecord, enableGallery: enableGallery, listener: listener));
    return dialog;
  }

  static Widget loadImage(String imageUrl, double width, double height, {BoxFit fit: BoxFit.cover, Widget placeHolder, Widget errorWidget}){
    final targetPlaceHolder = placeHolder != null ? placeHolder : Center(child: SizedBox(height: 30.0, width: 30.0, child: CircularProgressIndicator(strokeWidth: 2.0)));
    return CachedNetworkImage(
      imageUrl: imageUrl,
      width: width,
      height: height,
      fit: fit,
      placeholder: (context, url) => targetPlaceHolder,
      errorWidget: (context, url, error) => errorWidget,
    );
  }

  static Widget loadMediaImage(NUIMediaFile mediaFile, double width, double height, {BoxFit fit: BoxFit.cover, Widget placeHolder, Widget errorWidget}){
    final targetPlaceHolder = placeHolder != null ? placeHolder : Center(child: SizedBox(height: 30.0, width: 30.0, child: CircularProgressIndicator(strokeWidth: 2.0)));
    return CachedNetworkImage(
      imageUrl: mediaFile.path,
      width: width,
      height: height,
      fit: fit,
      placeholder: (context, url) => targetPlaceHolder,
      errorWidget: (context, url, error) => errorWidget,
    );
  }

  static ImageProvider loadImageProvider(String imageUrl){
    return CachedNetworkImageProvider(imageUrl);
  }

  static Widget loadImageLocal(String path, double width, double height, {BoxFit fit:BoxFit.cover}){
    return Image.file(File(path), width: width, height: height, fit: fit,);
  }

  static BetterPlayerController getVideoController(NUIMediaFile mediaFile, {BetterPlayerConfiguration config: const BetterPlayerConfiguration()}){
    if(mediaFile.isLocal){
      BetterPlayerDataSource betterPlayerDataSource = BetterPlayerDataSource(
          BetterPlayerDataSourceType.FILE,
          mediaFile.videoPath);
       return BetterPlayerController(
           BetterPlayerConfiguration(),
          betterPlayerDataSource: betterPlayerDataSource);
    }
    else{
      BetterPlayerDataSource betterPlayerDataSource = BetterPlayerDataSource(
          BetterPlayerDataSourceType.NETWORK,
          mediaFile.videoPath);

      return BetterPlayerController(
          config,
          betterPlayerDataSource: betterPlayerDataSource);
    }
  }

  static BetterPlayerDataSource dataSourceFromNUIMedia(NUIMediaFile media){
    return BetterPlayerDataSource(
          media.isLocal ? BetterPlayerDataSourceType.FILE : BetterPlayerDataSourceType.NETWORK,
          media.videoPath);
  }

  static List<BetterPlayerDataSource> dataSourceFromNUIMedias(List<NUIMediaFile> mediaFiles){
    List dataSourceList = List<BetterPlayerDataSource>();
    for(NUIMediaFile media in mediaFiles){
      dataSourceList.add(
        BetterPlayerDataSource(
          media.isLocal ? BetterPlayerDataSourceType.FILE : BetterPlayerDataSourceType.NETWORK,
          media.videoPath,
        ),
      );
    }
    return dataSourceList;
  }

}